'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: 
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 12/16/2007
'
' USAGE:  
' cscript Object_Attribute.vbs [DN | ADSPath | name]
'
' This script displays and allows you to change
' any single-valued attribute specified by sAttribute
' and accepts any name for an object:
'	User:		DN, ADSPath, SAMAccountName
'	Computer:	DN, ADSPath, computer name
'	Group:		DN, ADSPath, SAMAccountName
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit
Const ADS_PROPERTY_CLEAR = 1

Dim sObjectADsPath, oObject, sAttribute, sValue, sPrompt, sNewValue
Dim sDomainDN, sObjectName

'==========================================================================
' Configuration Block
' Define attribute to be used
sAttribute = "employeeNumber"
sDomainDN = "dc=contoso,dc=com"
'==========================================================================

If WScript.Arguments.Count = 0 or WScript.Arguments.Count > 1 Or (WScript.Arguments.Named.Exists("?")) Then
    WScript.Echo "Usage: cscript.exe " & WScript.ScriptFullName & _
         " <object DN | ADsPath | name>"
    WScript.Quit(0)
End If

sObjectName = WScript.Arguments(0)
' Translate name passed in argument, which could be
' sAMAccountName (user/group), name (computer), DN or ADsPath
' to an ADsPath
sObjectADsPath = ADObject_Find_Generic(sObjectName, sDomainDN)

On Error Resume Next
Set oObject = GetObject(sObjectADsPath)
If Err.Number <> 0 Then
    WScript.Echo "The object name provided is invalid:" & vbCrLf & _
        sObjectName & vbCrLf & "Error " & Err.Number & _
        vbCrLf & Err.Description
    WScript.Quit(0)
End If
On Error Goto 0

On Error Resume Next
sValue = oObject.Get(sAttribute)
If Err.Number <> 0 Then
    sPrompt = "There is no current value for " & sAttribute
    sValue = ""
Else
    sPrompt = "The current value for " & sAttribute & " is" & vbCrLf & _
               sValue
End If
On Error Goto 0
sNewValue = InputBox(sPrompt, sAttribute, sValue)

If sNewValue <> sValue Then
    On Error Resume Next
    If sNewValue > "" Then
        oObject.Put sAttribute, sNewValue
        oObject.SetInfo
    Else
    	If MsgBox("Do you want to delete the current value, " & sValue, vbYesNo, "Confirm deletion") = vbYes then
        	oObject.PutEx ADS_PROPERTY_CLEAR, sAttribute, 0
        	oObject.SetInfo
        End If
    End If
    If Err.Number <> 0 Then 
        WScript.Echo "Error changing value of " & sAttribute
        WScript.Quit(0)
    End If
End If

WScript.Quit(0)

'==========================================================================
' LIBRARY FUNCTIONS TO TRANSLATE ARGUMENT INTO ADSPATH
'==========================================================================

' #region Active Directory object find routines

Function ADObject_Find_Generic(ByVal sObject, ByVal sSearchDN)
	' Version 071130
	' Takes any input (name, DN, or ADsPath) of a user, computer, or group, and
	' returns the ADsPath of the object as a way of validating that the object exists
	'
	' INPUTS:	sObject 				DN or ADsPath to an object
	'					 				sAMAccountName (pre-Windows 2000 logon name) of a user or group
	'					 				computer name of a computer
	'			sSearchDN 				the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)
	'
	' RETURNS:	ADObject_Find_Generic 	ADsPath (LDAP://...) of the object
	'								   	blank if object was not found
	'
	' NOTES:	ASSUMPTION: computers, users & groups have unique names. See note inline.
	'
	' REQUIRES	AD_Search_Array routine
	'			AD_Search_RS routine
	' 			ADObject_Validate routine
	
	Dim aResults, sLDAPQuery
	Select Case ADObject_NameType(sObject)
		Case ""
			ADObject_Find_Generic = ""
		Case "adspath"
			ADObject_Find_Generic = ADObject_Validate(sObject)
		Case "distinguishedname"
			ADObject_Find_Generic = ADObject_Validate("LDAP://" & sObject)
		Case "name"
				' Assumption: No computer has the same name as a user's or group's sAMAccountName
				' otherwise, this query will return more than one result
				sLDAPQuery = "<LDAP://" & sSearchDN & ">;" & _
							 "(|(samAccountName=" & sObject & ")(samAccountName=" & sObject & "$));" & _
							 "aDSPath;subtree"
				aResults = AD_Search_Array (sLDAPQuery)
				If Ubound(aResults) = -1 Then
					ADObject_Find_Generic = ""
				Else
					ADObject_Find_Generic = aResults(0)
				End If
	End Select
	
End Function

Function ADObject_NameType(ByVal sObjectName)
	' Version 071204
	' Evaluates sObjectName to determine what type of name it is
	' Returns	ADObject_NameType	adspath
	'								distinguishedname
	'								name
	'								blank if sObjectName = ""
	
	Dim sNameType
	
	If Len(sObjectName) = 0 Then
		sNameType = ""

	ElseIf Len(sObjectName) < 3 Then
		' can't be a DN or an ADsPath - must be a name
		sNameType = "name"
	
	ElseIf Ucase(Left(sObjectName,3)) = "CN=" Then
		' is a DN
		sNameType = "distinguishedname"
		
	ElseIf Len(sObjectName) < 8 Then
		' too short to be an ADsPath and isn't a DN, so it must be a name
		sNameType = "name"
		
	ElseIf UCase(Left(sObjectName, 7)) = "LDAP://" Then
		' is already an ADsPath
		sNameType = "adspath"

	Else
		' must be a name
		sNameType = "name"

	End If

	ADObject_NameType = sNameType
End Function

Function ADObject_Validate(ByVal sObjectADsPath)
	' Version 071122
	' Returns ADsPath of object as a way of validating that the object exists
	'
	' INPUTS:	sObjectADsPath		ADsPath of object to test
	' RETURNS:	ADObject_Validate	Path of object (if it exists) or blank
	
	Dim oObject
	On Error Resume Next
	Set oObject = GetObject(sObjectADsPath)
	If Err.Number <> 0 Then
		ADObject_Validate = ""
		Err.Clear
	Else
		ADObject_Validate = oObject.ADsPath
	End If	
End Function

Function AD_Search_Array(sLDAPQuery)
	' Version 071130
	' Returns an array of AD objects
	' A wrapper around AD_Search_RS
	' Inputs:	sLDAPQuery		The query to perform
	' Returns	AD_Search_Array	Array of Field(0) of recordset
	'							or empty array if recordset is empty
	
	Dim rs, aResults
	aResults = array()
	Set rs = AD_Search_RS(sLDAPQuery)
	If Not(rs Is Nothing) Then
		If Not (rs.BOF And rs.EOF) Then
			rs.MoveFirst
			Do While Not rs.EOF
				Redim Preserve aResults(ubound(aResults) + 1)
				aResults(ubound(aResults)) = rs.Fields(0)
				rs.MoveNext
			Loop
		End If
	Else
		' Error handling: query returned Nothing
	End If
	AD_Search_Array = aResults
	
End Function

Function AD_Search_RS(sLDAPQuery)
	' Version 071130
	' Returns a recordset of AD objects
	' Inputs:	sLDAPQuery		The query to perform
	' Returns:	AD_Search_RS	Recordset
	
	Dim oConnection
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
		Set AD_Search_RS = Nothing
	Else
		Set AD_Search_RS = oConnection.Execute(sLDAPQuery)
	End If
End Function

' #endregion
