'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: 
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 12/16/2007
'
' USAGE:  
' cscript.exe Schema_Create_AssignedComputers.vbs
'
' Demonstrates the creation of a pair of attributes and object classes
' that allow you to associate one or more computers with one or more users
' The script creates a forward link attribute called Computer-Assigned-To
' and a back link attribute called Assigned-Computers.
' You must then (manually) create object classes that include
' these attributes (e.g. UserInfo and ComputerInfo) and associate the
' object classes with the User and Computer object classes.
'
' The script requires customization, including the creation of your own
' root OID (using OIDGen.vbs).
'
' Be sure to read all comments in the script.
' Because this script modifies the schema there's a SAFETY SWITCH
' in the script that will PREVENT THE SCRIPT FROM RUNNING BY DEFAULT.
' It's up to you to understand and prepare for a test of the script in 
' your non-production environment before turning off the safety switch.
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit
Dim oAttribute
Dim oSchema
Dim sAttributeADsPath
Dim oRootDSE
Dim sForestRootDN, sAttributeID, sAttributeCN, sDescription
Dim iOMSyntax, sAttributeSyntax, iSearchFlags, bIsSingleValued, sLinkID
Dim bSafetySwitch

'==========================================================================
' SCRIPT CONFIGURATION BLOCK
'==========================================================================
bSafetySwitch = True	' If TRUE, script will not run. 
						' Used to prevent accidental schema changes in production
						' Change to FALSE when you are ready to test the script in a lab
sForestRootDN = "dc=contoso,dc=com"

'==========================================================================
' FORWARD LINK CONFIGURATION BLOCK
'
' AttributeID: use OIDGen.vbs to create the root OID, then add .2 for attributes
' and then .x for each successive new attribute. The example below uses a root
' OID obtained by running OIDGen.vbs and it is the first new attribute (.2.1)
' Be sure to use the root OID that you receive when yhou run OIDGen.vbs
sAttributeID = "1.2.840.113556.1.8000.2554.999999.2.1"
sAttributeCN = "contoso-computerAssignedTo"
sDescription = "Computer-Assigned-To"
' See http://msdn2.microsoft.com/en-us/library/ms684419.aspx for the OM ID and Syntax ID
' for the type of attribute you're trying to create
' For a Unicode String (similar to many of the standard attributes)
'    iOMSyntax = 		27
'    sAttributeSyntax =	"2.5.5.3"
' For a DN
'	iOMSyntax =			127
'	sAttributeSyntax =	"2.5.5.1"
iOMSyntax = 127
sAttributeSyntax = "2.5.5.1"
' iSearchFlags  see http://msdn2.microsoft.com/en-us/library/ms679765.aspx
' 1 = Index this attribute; 0 = Do not
iSearchFlags = 0
' True = single valued; False = multi-valued
bIsSingleValued = False
' If NOT creating a linked attribute, set
' sLinkID = ""
' If creating a forward link, use
' sLinkID = "1.2.840.113556.1.2.50"
' If creating a back link (be sure you reloaded the schema prior to creating it),
' sLinkID = the AttributeID of the forward link
sLinkID = "1.2.840.113556.1.2.50"
'==========================================================================

If bSafetySwitch Then
	WScript.Echo "This script should not be tested in a production network."
	WScript.Echo "Be sure to inform yourself as to the impact, concerns, "
	WScript.Echo "and issues surrounding schema extension."
	WScript.Echo "When you are ready to test this script in a lab environment,"
	WScript.Echo "change the Configuration Block setting for the variable"
	WScript.Echo "bSafetySwitch to False."
	WScript.Quit
End If

'--------------------------------------------------------------------------
' Create the forward link attribute

Set oSchema = _
  GetObject("LDAP://cn=Schema,cn=Configuration," & sForestRootDN)
   
Set oAttribute = oSchema.Create("attributeSchema", "cn=" & sAttributeCN)
   
' Mandatory attributes
oAttribute.Put "lDAPDisplayName", sAttributeCN
oAttribute.Put "attributeId", sAttributeID
oAttribute.Put "oMSyntax", iOMSyntax
oAttribute.Put "attributeSyntax", sAttributeSyntax
oAttribute.Put "isSingleValued", bIsSingleValued
oAttribute.Put "adminDescription", sDescription

If sLinkID > "" Then oAttribute.Put "linkID", sLinkID

' Commit change
oAttribute.SetInfo
   
' Optional attributes
oAttribute.GetInfo
oAttribute.Put "searchFlags", iSearchFlags
oAttribute.SetInfo

'--------------------------------------------------------------------------
' Retrieve all the information as a report of success

sAttributeADsPath = "LDAP://cn=" & sAttributeCN & ",cn=Schema,cn=configuration," & sForestRootDN
WScript.Echo
WScript.Echo "CONFIRMATION OF NEWLY ADDED ATTRIBUTE:"
WScript.Echo sAttributeADsPath
Set oAttribute = GetObject(sAttributeADsPath)
On Error Resume Next ' in case a property does not exist like LinkID
WScript.Echo "lDAPDisplayname: " & oAttribute.lDAPDisplayName
WScript.Echo "description:     " & oAttribute.adminDescription
WScript.Echo "Attribute ID:    " & oAttribute.attributeID
WScript.Echo "oMSyntax:        " & oAttribute.oMSyntax
WScript.Echo "attributeSyntax: " & oAttribute.attributeSyntax
WScript.Echo "isSingleValued:  " & oAttribute.isSingleValued
WScript.Echo "searchFlags:     " & oAttribute.searchFlags
WScript.Echo "linkID:          " & oAttribute.linkID
On Error Goto 0
WScript.Echo

'--------------------------------------------------------------------------
' Reload the schema

If sLinkID = "1.2.840.113556.1.2.50" Then 
	WScript.Echo "Forward link created: Reloading the schema."
	Set oRootDSE = GetObject("LDAP://RootDSE")
	oRootDSE.Put "schemaUpdateNow", 1
	oRootDSE.SetInfo
	WScript.Echo "The Schema has been reloaded"
End If

'==========================================================================
' BACK LINK ATTRIBUTE CONFIGURATION BLOCK
'
sAttributeID = "1.2.840.113556.1.8000.2554.999999.2.2"
sAttributeCN = "contoso-assignedComputers"
sDescription = "Assigned-Computers"
iOMSyntax = 127
sAttributeSyntax = "2.5.5.1"
iSearchFlags = 0
bIsSingleValued = False
' If creating a back link (be sure you reloaded the schema prior)
' sLinkID = the AttributeID or ldapDisplayName of the forward link
sLinkID = "contoso-computerAssignedTo"
'==========================================================================

'--------------------------------------------------------------------------
' Create the back link attribute

Set oSchema = _
  GetObject("LDAP://cn=Schema,cn=Configuration," & sForestRootDN)
   
Set oAttribute = oSchema.Create("attributeSchema", "cn=" & sAttributeCN)
   
' Mandatory attributes
oAttribute.Put "lDAPDisplayName", sAttributeCN
oAttribute.Put "attributeId", sAttributeID
oAttribute.Put "oMSyntax", iOMSyntax
oAttribute.Put "attributeSyntax", sAttributeSyntax
oAttribute.Put "isSingleValued", bIsSingleValued
oAttribute.Put "adminDescription", sDescription

If sLinkID > "" Then 
	oAttribute.Put "linkID", sLinkID
End If

' Commit change
oAttribute.SetInfo
   
' Optional attributes
oAttribute.GetInfo
oAttribute.Put "searchFlags", iSearchFlags
oAttribute.SetInfo

'--------------------------------------------------------------------------
' Retrieve all the information as a report of success

sAttributeADsPath = "LDAP://cn=" & sAttributeCN & ",cn=Schema,cn=configuration," & sForestRootDN
WScript.Echo
WScript.Echo "CONFIRMATION OF NEWLY ADDED ATTRIBUTE:"
WScript.Echo sAttributeADsPath
Set oAttribute = GetObject(sAttributeADsPath)
On Error Resume Next ' in case a property does not exist like LinkID
WScript.Echo "lDAPDisplayname: " & oAttribute.lDAPDisplayName
WScript.Echo "description:     " & oAttribute.adminDescription
WScript.Echo "Attribute ID:    " & oAttribute.attributeID
WScript.Echo "oMSyntax:        " & oAttribute.oMSyntax
WScript.Echo "attributeSyntax: " & oAttribute.attributeSyntax
WScript.Echo "isSingleValued:  " & oAttribute.isSingleValued
WScript.Echo "searchFlags:     " & oAttribute.searchFlags
WScript.Echo "linkID:          " & oAttribute.linkID
On Error Goto 0

WScript.Echo
WScript.Echo "COMPLETED. Review above for any signs of errors"
WScript.Echo "If you created a forward and back link attribute pair,"
WScript.Echo "the LinkID of the attributes should be only one number apart."
