'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: User_Provision.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 12/16/2007
'
' Provisions a user account, demonstrating business logic in the
' determination of name properties
' 
' USAGE
' createuser.vbs  
' 	/firstname:John [/initial:Q] [/middlename:Franklin] /lastname:Doe
'	/samaccountname:"jdoe"
'	/ou:"ou=Employees,ou=People,dc=contoso,dc=com" 
'	[/description:"Accountant in Sydney"]
'	
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
' You are free to use this script and make derivatives of it,
' but you may not remove any of this header.
' We also ask that any functional improvements be reported to
' Intelliem at the URL above for the benefit of all.
'==========================================================================

Option Explicit
Dim bInteractive
Dim sDomainDNS, sDomainNetBIOS, sDomainDN
Dim iPasswordLength, sUPNSuffix
Dim sSAM, sUPN, sCN, sDisplay, sUser
Dim sFirst, sInit, sMiddle, sLast
Dim sDescription, sTargetOU
Dim sProfileNamespace, sProfilePath

' ============================================================================
' CONFIGURATION BLOCK
bInteractive = True		' TRUE:  Script can be run from command prompt.
						' FALSE: Limits the output of the script (for running unattended).
sDomainDNS = "contoso.com"
sDomainDN = "dc=contoso,dc=com"
sDomainNetBIOS = "CONTOSO"
sUPNSuffix = "contoso.com"
sProfileNamespace = "\\contoso.com\users\%username%\profile"
iPasswordLength = 15
' ============================================================================

Call MainRoutine()	' Code is contained in a subroutine to make it easier to port to an HTA
WScript.Quit

Sub MainRoutine()
	Dim oUser, oTargetOU

	Call Arguments()				' Take what was passed to the script
	Call BusinessLogic()			' Apply business logic
	If Not Validate() Then Exit Sub	' See if the results are valid, and if not, quit
		
	' Connect to OU
	Set oTargetOU = GetObject("LDAP://" & sTargetOU)
	' Create user and properties
	Set oUser = oTargetOU.Create("user","cn=" & sCN)
	WriteProperty oUser,"userPrincipalName", sUPN
	WriteProperty oUser,"sAMAccountName", sSAM
	WriteProperty oUser,"givenName", sFirst
	WriteProperty oUser,"initials", sInit
	WriteProperty oUser,"middleName", sMiddle
	WriteProperty oUser,"sn", sLast
	WriteProperty oUser,"description", sDescription
	WriteProperty oUser,"displayName", sDisplay
	WriteProperty oUser,"profilePath", sProfilePath
	oUser.SetInfo
	
	' Generate random password
	Dim sTempPassword, iCase, i
	' Start with an upper, a lower, and a numeric character
	' to ensure complexity requirements are met
	sTempPassword = RandomLetter(1)
	sTempPassword = RandomLetter(2)
	sTempPassword = RandomLetter(3)
	' Now fill the remaining length with random characters
	For i=1 To iPasswordLength - 3
		randomize()
		iCase = int(3*rnd())
		sTempPassword = sTempPassword & RandomLetter(iCase)
	Next
	oUser.SetPassword sTempPassword
	
	' Enable account
	oUser.AccountDisabled = False
	oUser.SetInfo
	
	WScript.Echo "User " & sCN & " created."
End Sub

Sub	Arguments()
	Dim oArgs

	' Check for "HELP"
	If (WScript.Arguments.Count = 0) Or _
		(WScript.Arguments.Named.Exists("?")) Then Call Usage()

	Set oArgs = WScript.Arguments.Named
	sFirst = oArgs("firstname")
	sInit = oArgs("initial")
	sMiddle = oArgs("middlename")
	sLast = oArgs("lastname")
	sSAM = oArgs("samaccountname")
	sDescription = oArgs("description")
	sTargetOU = oArgs("ou")
	
End Sub

Sub BusinessLogic()
	Call CalculateNames()
	sProfilePath = Replace(sProfileNamespace, "%username%", sSAM)
End Sub

Sub CalculateNames()
	Dim sName, sNamePart
	
	' COMMON NAME
	sName = ""
	sNamePart = sFirst
	If sNamePart > "" Then sName = sName & sNamePart & " "
	sNamePart = sMiddle
	If sNamePart > "" Then sName = sName & sNamePart & " "
	sNamePart = sLast
	If sNamePart > "" Then sName = sName & sNamePart & " "
	' Trim off the last space
	sName = Trim(sName)
	sCN = sName
	
	' DISPLAY NAME
	sName = ""
	sNamePart = sLast
	If sNamePart > "" Then sName = sName & sNamePart
	If sFirst > "" Or sMiddle > "" Then
		sName = sName & ", "
	End If
	sNamePart = sFirst
	If sNamePart > "" Then sName = sName & sNamePart & " "
	sNamePart = sMiddle
	If sNamePart > "" Then sName = sName & sNamePart & " "
	' Trim off the last space
	sName = Trim(sName)
	sDisplay = sName
	
	' USER PRINCIPAL NAME (email: first.middle.last)
	sName = ""
	sNamePart = sFirst
	If sNamePart > "" Then sName = sName & sNamePart & " "
	sNamePart = sMiddle
	If sNamePart > "" Then sName = sName & sNamePart & " "
	sNamePart = sLast
	If sNamePart > "" Then sName = sName & sNamePart & " "
	' Trim off the last space
	sName = Trim(sName)
	' Replace spaces with periods
	sName = Replace(sName, " ", ".")
	' Add UPN suffix
	sName = sName & "@" & sUPNSuffix
	sUPN = sName
	
End Sub

Function Validate()
	If (sUPN="") Or (sSAM="") Or (sCN="") Or (sDisplay="") Or (sTargetOU="") Then
		WScript.Echo "Required argument(s) missing."
		Validate = False
	Else
		Validate = True
	End If
End Function

Sub Usage()
	If bInteractive Then
		WScript.Echo
		WScript.Echo "USER_CREATE.VBS"
		WScript.Echo "Creates a user object"
		WScript.Echo "Usage: cscript.exe  " & WScript.ScriptName & VbCrLf & _
					 "                    /firstname:John [/initial:Q] [/middlename:Franklin]" & VbCrLf & _
					 "                    /lastname:Doe" & VbCrLf & _
					 "                    /samaccountname:""jdoe""" & VbCrLf & _
					 "                    /ou:""ou=Employees,ou=People,dc=contoso,dc=com""" & VbCrLf & _
					 "                    [/description:""Accountant in Sydney""]" & VbCrLf
	End If
	WScript.Quit(500)
End Sub

Sub WriteProperty(oDirectoryEntry, sPropertyName, sPropertyValue)
   If sPropertyValue>"" then
       oDirectoryEntry.Put sPropertyName,sPropertyValue
   End If
End Sub

Function RandomLetter(iCase)
  ' Returns a random letter or number
  ' icase = 0 (UPPER), 1 (lower) or 2 (numeric)
  Dim iMax, iMin
  randomize()
  Select Case iCase
  Case 0
    iMin=65
    iMax=26
  Case 1
    iMin=97
    iMax=26
  Case 2
    iMin=48
    iMax=10
  End Select
  RandomLetter=chr( int( iMax*rnd() + iMin) )
End Function

