'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: groupaddmember.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 12/06/2007
'
' USAGE:  
' cscript groupaddmember.vbs 
'					  /groupadspath:"LDAP://CN=..." 
'					  /memberadspath:"LDAP://CN=..." 
'                     /justification:"business justification" 
'                     /requestedby:"DOMAIN\username"
'
' VALIDATION: User specified by requestedBy must be 
' in the group's managedBy attribute
'
' Adds a user to a group. To be used by the Proxy Framework.
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit

' DERIVE DOMAIN INFORMATION
Dim adSys, sComputerDN, sDomainDNS, sDomainNetBIOS, sDomainDN
Set adSys = CreateObject("ADSystemInfo")
sComputerDN = adSys.ComputerName
sDomainDN = Mid(sComputerDN, InStr(lcase(sComputerDN),",dc=") +1 )
sDomainDNS = adSys.DomainDNSName
sDomainNetBIOS = adSys.DomainShortName

Dim bInteractive
' ============================================================================
' CONFIGURATION BLOCK
bInteractive = True		' TRUE:  Script can be run from command prompt.
						' FALSE: Limits the output of the script (for running unattended).
' ============================================================================

' Decalare variables for arguments
Dim sGroupADsPath, sMemberADsPath, sJustification, sRequestorSAM, sRequestorDN, oGroup, oUser, oRequestor

Call MainRoutine()	' Code is contained in a subroutine to make it easier to port to an HTA
WScript.Quit

Sub MainRoutine()
	Call Arguments()				' Take what was passed to the script
	Call BusinessLogic()			' Apply business logic
	If Not Validate() Then WScript.Quit(500)
	If Not Validate_Requestor() Then WScript.Quit(500)
	If Group_AddMember(sGroupADsPath, sMemberADsPath) Then
		WScript.Echo "Successfully added member to group."
		WScript.Echo "Group:  " & sGroupADsPath
		WScript.Echo "Member: " & sMemberADsPath
	Else
		WScript.Echo "Failed to add member to group."
		WScript.Echo "Group:  " & sGroupADsPath
		WScript.Echo "Member: " & sMemberADsPath		
		WScript.Quit(500)		
	End If
End Sub

Sub Arguments()
	Dim oArgs

	' Check for "HELP"
	If (WScript.Arguments.Count = 0) Or _
		(WScript.Arguments.Named.Exists("?")) Then Call Usage()

	Set oArgs = WScript.Arguments.Named
	sGroupADsPath = oArgs("groupadspath")
	sMemberADsPath = oArgs("memberadspath")
	sJustification = oArgs("justification")
	sRequestorSAM = oArgs("requestedBy")
	
End Sub

Sub BusinessLogic()
End Sub

Function Validate()
	Validate = False
	If (sGroupADsPath="") Then 
		WScript.Echo "sGroupADsPath missing."
		Exit Function
	End If
	If (sMemberADsPath="") Then 
		WScript.Echo "sMemberADsPath missing."
		Exit Function
	End If
	If (sJustification="") Then 
		WScript.Echo "sJustification missing."
		Exit Function
	End If
	If (sRequestorSAM="") Then 
		WScript.Echo "sRequestorSAM missing."
		Exit Function
	End If
	Set oGroup = ADObject_Get_Generic(sGroupADsPath, sDomainDN)
	If oGroup Is Nothing Then
		WScript.Echo "Group ADsPath invalid."
		Exit Function
	End If
	Set oUser = ADObject_Get_Generic(sMemberADsPath, sDomainDN)
	If oUser Is Nothing Then
		WScript.Echo "User ADsPath invalid."
		Exit Function
	End If
	Validate = True
End Function

Function Validate_Requestor()
	Dim dMembers
	Dim sTemp
	Validate_Requestor = False
	Dim sManagedBy, oManagedBy
	
	' Get requestor's DN
	sRequestorSAM = Replace(sRequestorSAM, sDomainNetBIOS & "\", "")
	sRequestorDN = Replace(ADObject_Find_Generic(sRequestorSAM, sDomainDN), "LDAP://", "")
	Set oRequestor = ADObject_Get_Generic(sRequestorDN, sDomainDN)
	If oRequestor Is Nothing Then
		WScript.Echo "RequestedBy invalid."
		Exit Function
	End If
		' Get ManagedBy attribute
	On Error Resume Next
	sManagedBy = oGroup.Get("managedBy")
	On Error GoTo 0
	If sManagedBy = "" Then 
		WScript.Echo "ManagedBy attribute of the group not specified."
		Exit Function
	End If
	
	' Is the requestor specified as the ManagedBy account?
	If sManagedBy = sRequestorDN Then
		Validate_Requestor = True
		Exit Function
	End If

	' Does the requestor belong to the ManagedBy account?
	Set dMembers = Members_Get(sManagedBy)
	If Not (dMembers.Exists(sRequestorDN)) Then
		WScript.Echo "Requestor does not belong to ManagedBy account."
		Exit Function
	End If

	Validate_Requestor = True
		
End Function

Sub Usage()
	If bInteractive Then
		WScript.Echo
		WScript.Echo "GROUP_ADDMEMBER.VBS"
		WScript.Echo "Adds a member to a group"
		WScript.Echo "Usage: cscript.exe  " & WScript.ScriptName & VbCrLf & _
					 "                    /groupadspath:""LDAP://CN=...""" & VbCrLf & _
					 "                    /memberadspath:""LDAP://CN=...""" & VbCrLf & _
					 "                    /justification:""business justification""" & VbCrLf & _
					 "                    /requestedby:""cn=user name,ou=...,dc=contoso,dc=com""" & VbCrLf
	End If
	WScript.Quit(500)
End Sub

' #region member enumeration

Function Members_Get(ByVal sGroupDN)
	' Version 071204
	' Enumerates the members of a group including nested members
	' Does NOT consider users who are assigned the group as their primary group
	' because of the performance penalty of calculating PrimaryGroupTokens
	' INPUTS:	sObjectDN: 		the DN of the group
	' RETURNS:	Members_Get 	dictionary object of members, sorted
	'				Key  		distinguishedName of member
	'				Item		LEVEL:	0:	direct member
	'									X: 	first level at which the member was encountered
	' REQUIRES:	Member_Enum routine
	' USAGE:	Dim dMembersList
	'			Set dMembersList = Members_Get("cn=managers,ou=groups,dc=contoso,dc=com")
	
	Dim oObject
	Dim aMembers
	
	Dim dMembers
	Set dMembers = CreateObject("Scripting.Dictionary")
	dMembers.CompareMode = vbTextCompare
		
	' Create an object reference to the specified group
	' and trap errors that might arise if the object does not exist
	On Error Resume Next
	Set oObject = GetObject("LDAP://" & sGroupDN)
	If Err.Number<>0 Then
	    'Handle error: Group not found
	    Err.Clear
	    Set Members_Get = dMembers
	    Exit Function
	End If
	On Error GoTo 0 
	
	Call Members_Enum(oObject, 0, dMembers)
	Set Members_Get = dMembers
	
End Function

Sub Members_Enum(ByVal oObject, ByVal iLevel, ByRef dMembers)
    ' Version 071204
    ' Enumerates member property of a group
    ' INPUTS:	oObject 		group for which to enumerate Members
    '			iLevel			the nesting level - call routine with iLevel = 0
    '			dMembers		dictionary object
	' RESULTS:	dMembers		dictionary object
	'				Key  		distinguishedName of member
	'				Item		LEVEL:	0: direct member
	'									X: first level at which the member was encountered
	' USAGE:	Dim dMembers
	'			Set dMembers = CreateObject("Scripting.Dictionary")
	'			dMembers.CompareMode = vbTextCompare
	'			Dim oObject
	'			Set oObject = GetObject("LDAP://cn=managers,ou=groups,dc=contoso,dc=com")
	'			Call Members_Enum(oObject, 0, dMembers)
	' NOTE:		Recommend using Members_Get to call
	
	Dim aMembers
	Dim sMember
	Dim oMember
	Dim sMemberSAM
	
	On Error Resume Next
	If oObject.class <> "group" Then
		Exit Sub
	End If
	
	aMembers = oObject.GetEx("member")
	If Err.Number <> 0 Then
		' Assume error is because member has no distinguished names
		Err.Clear
		On Error GoTo 0
		Exit Sub
	Else
		On Error GoTo 0
		For Each sMember In aMembers
			If (dMembers.Exists(sMember) = False) Then
				' We have not seen this member yet
				' Add it to the dictionary object
				dMembers.Add sMember, iLevel
				' Create an object reference to the member
				Set oMember = GetObject("LDAP://" & sMember)
				' Enumerate this group's nested members
				Call Members_Enum(oMember, iLevel+1, dMembers)
			ElseIf dMembers.Item(sMember) > iLevel Then
				' We have seen this member, but "further away"
				' Keep track of the most direct level
				dMembers.Item(sMember) = iLevel
			End If
	    Next
	End If

End Sub

' #endregion

' #region Active Directory object find routines

Function ADObject_Get_Generic(ByVal sObject, ByVal sSearchDN)
	' Version 071204
	' Takes any input (name, DN, or ADsPath) of a user, computer, or group, And
	' returns an object reference to the object, or nothing if it does not exist
	' This is a wrapper around ADObject_Find_Generic.
	' Requires ADObject_Find_Generic.
	' See notes for ADObject_Find_Generic
	
	Dim oADObject, sADsPath
	sADsPath = ADObject_Find_Generic(sObject, sSearchDN)
	If sADsPath > "" Then
		Set ADObject_Get_Generic = GetObject(sADsPath)
	Else
		Set ADObject_Get_Generic = Nothing
	End If
	
End Function

Function ADObject_Find_Generic(ByVal sObject, ByVal sSearchDN)
	' Version 071130
	' Takes any input (name, DN, or ADsPath) of a user, computer, or group, and
	' returns the ADsPath of the object as a way of validating that the object exists
	'
	' INPUTS:	sObject 				DN or ADsPath to an object
	'					 				sAMAccountName (pre-Windows 2000 logon name) of a user or group
	'					 				computer name of a computer
	'			sSearchDN 				the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)
	'
	' RETURNS:	ADObject_Find_Generic 	ADsPath (LDAP://...) of the object
	'								   	blank if object was not found
	'
	' NOTES:	ASSUMPTION: computers, users & groups have unique names. See note inline.
	'
	' REQUIRES	AD_Search_Array routine
	'			AD_Search_RS routine
	' 			ADObject_Validate routine
	
	Dim aResults, sLDAPQuery
	Select Case ADObject_NameType(sObject)
		Case ""
			ADObject_Find_Generic = ""
		Case "adspath"
			ADObject_Find_Generic = ADObject_Validate(sObject)
		Case "distinguishedname"
			ADObject_Find_Generic = ADObject_Validate("LDAP://" & sObject)
		Case "name"
				' Assumption: No computer has the same name as a user's or group's sAMAccountName
				' otherwise, this query will return more than one result
				sLDAPQuery = "<LDAP://" & sSearchDN & ">;" & _
							 "(|(samAccountName=" & sObject & ")(samAccountName=" & sObject & "$));" & _
							 "aDSPath;subtree"
				aResults = AD_Search_Array (sLDAPQuery)
				If Ubound(aResults) = -1 Then
					ADObject_Find_Generic = ""
				Else
					ADObject_Find_Generic = aResults(0)
				End If
	End Select
	
End Function

Function ADObject_NameType(ByVal sObjectName)
	' Version 071130
	' Evaluates sObjectName to determine what type of name it is
	' Returns	ADObject_NameType	adspath
	'								distinguishedname
	'								name
	'								blank if sObjectName = ""
	
	Dim sNameType
	
	If Len(sObjectName) = 0 Then
		sNameType = ""

	ElseIf Len(sObjectName) < 3 Then
		' can't be a DN or an ADsPath - must be a name
		sNameType = "name"
	
	ElseIf lcase(Left(sObjectName,3)) = "cn=" Then
		' is a DN
		sNameType = "distinguishedname"
		
	ElseIf Len(sObjectName) < 8 Then
		' too short to be an ADsPath and isn't a DN, so it must be a name
		sNameType = "name"
		
	ElseIf ucase(Left(sObjectName, 7)) = "LDAP://" Then
		' is already an ADsPath
		sNameType = "adspath"

	Else
		' must be a name
		sNameType = "name"

	End If

	ADObject_NameType = sNameType
End Function

Function ADObject_Validate(ByVal sObjectADsPath)
	' Version 071122
	' Returns ADsPath of object as a way of validating that the object exists
	'
	' INPUTS:	sObjectADsPath		ADsPath of object to test
	' RETURNS:	ADObject_Validate	Path of object (if it exists) or blank
	
	Dim oObject
	On Error Resume Next
	Set oObject = GetObject(sObjectADsPath)
	If Err.Number <> 0 Then
		ADObject_Validate = ""
		Err.Clear
	Else
		ADObject_Validate = oObject.ADsPath
	End If	
End Function

Function AD_Search_Array(sLDAPQuery)
	' Version 071130
	' Returns an array of AD objects
	' Inputs:	sLDAPQuery		The query to perform
	' Returns	AD_Search_Array	Array of Field(0) of recordset
	'							or empty array if recordset is empty
	
	Dim rs, aResults
	aResults = array()
	Set rs = AD_Search_RS(sLDAPQuery)
	If Not(rs Is Nothing) Then
		If Not (rs.BOF And rs.EOF) Then
			rs.MoveFirst
			Do While Not rs.EOF
				Redim Preserve aResults(ubound(aResults) + 1)
				aResults(ubound(aResults)) = rs.Fields(0)
				rs.MoveNext
			Loop
		End If
	Else
		' Error handling: query returned Nothing
	End If
	AD_Search_Array = aResults
	
End Function

Function AD_Search_RS(sLDAPQuery)
	' Version 071130
	' Returns a recordset of AD objects
	' Inputs:	sLDAPQuery		The query to perform
	' Returns:	AD_Search_RS	Recordset
	
	Dim oConnection
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
		Set AD_Search_RS = Nothing
	Else
		Set AD_Search_RS = oConnection.Execute(sLDAPQuery)
	End If
End Function

' #endregion

' #region Group membership changes

Function Group_AddMember(ByVal sGroupADsPath, ByVal sMemberADsPath)
	' Version 071130
	Dim oGroup, oMember
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> 0 Then
		' Handle error: could not connect to group
		WScript.Echo "Error connecting to group."
		Group_AddMember = False
		Err.Clear
		Exit Function
	End If
	oGroup.Add sMemberADsPath
	If Err.Number > 0 then
		' Handle error: could not add member
		WScript.Echo "Error adding member: " & Err.Description
		Group_AddMember = False
		Err.Clear
	Else
		Group_AddMember = True
	End If
	On Error Goto 0
End Function

Function Group_RemoveMember(ByVal sGroupADsPath, ByVal sMemberADsPath)
	' Version 071130
	Dim oGroup, oMember
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> 0 Then
		' Handle error: could not connect to group
		WScript.Echo "Error connecting to group."
		Group_RemoveMember = False
		Err.Clear
		Exit Function
	End If
	oGroup.Remove sMemberADsPath
	If Err.Number > 0 then
		' Handle error: could not remove member
		WScript.Echo "Error removing member: " & Err.Description
		Group_RemoveMember = False
		Err.Clear
	Else
		Group_RemoveMember = True
	End If
	On Error Goto 0
End Function

' #endregion