'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Config_QuickLaunch_Toggle.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/21/2007
'
' USAGE:  
' cscript.exe Config_QuickLaunch_Toggle.vbs
'
' Toggles the QuickLaunch bar
'
' Demonstrates how to use SendKeys and AppActivate to automate
' a GUI click-by-click (keystroke-by-keystroke) configuration change
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit
Dim WSHShell, iObserve, iWait, sMessage, iButton, temp
' iWait is the standard delay used to wait for Explorer to make a change,
' such as open a window or application. You can lengthen or shorten
' this delay (in MILLISECONDS) based on your testing
' If you create your own automation based on this script, use 
' wscript.sleep iWait to create delays that you always want included
iWait=1000
' iObserve is a delay that's used to allow the macro to be 'observed'
' by the user -- so that changes don't just blast by.
' You can set this to zero and it will not have an impact on the
' script's functionality
iObserve=2000

' ==============================================
' SET TASKBAR TO SHOW QUICKLAUNCH
' ==============================================

Set WSHShell = WScript.CreateObject("WScript.Shell")

' in the event that the script is run by a user, 
' notify the user to let the script run without interference.
' the POPUP method is used so that the message box will disappear
' and the script will continue in an unattended fashion
sMessage = "A 'macro' will run to " & _
  "configure the Quick Launch bar. " & _
  "It is VERY important that you do not touch your system until the " & _
  "macro is complete.  Click OK to begin the macro, then wait."
iButton = WSHShell.Popup (sMessage, 5, "Macro Notice", 64)

' Open the Taskbar and Start Menu Properties dialog
WSHShell.Run "rundll32.exe shell32.dll,Options_RunDLL 1"
WScript.Sleep iWait
temp = ActivateWindow("Taskbar and Start Menu Properties", 5, True)
WScript.Sleep iWait

' Send the keystrokes to toggle Quick Launch visibility
WSHShell.SendKeys "%Q"
WScript.Sleep iObserve
WSHShell.SendKeys "{ENTER}"

' Notify the user that the script is comlete
' Use the POPUP method so that the script can complete
' if it is running unattended
sMessage = "Thank you. The macro is complete. Click OK to continue."
iButton = WSHShell.Popup (sMessage, 2, "Macro Notice", 64)

Function ActivateWindow(sWindow, iTimeout, bQuit)
	' Version 071121
	'
	' This function is a wrapper around the
	' Shell object's AppActivate method
	' sWindow:	The name of the window to activate
	'			can be the full name or a partial string that is
	'			at the beginning or end of the window title
	' iTimeout:	The time (in SECONDS) to wait for the window to appear
	' bQuit:	TRUE: causes the entire (parent) script to exit if the window
	'			cannot be activated.
	'			FALSE: return to script processing
	' RETURNS:	ActivateWindow() will be TRUE if sWindow was successfully activated
	'			and FALSE if it was not (and if the script did not quit)
	
	Dim time
	time = now()
	
	' Attempt to activate the window every 200 milliseconds
	' Until the timeout is reached
	While WSHShell.AppActivate(sWindow) = False AND DateDiff("s", time, now) < iTimeout
	  wscript.sleep 200
	Wend
	
	' Try activating it one more time, now that the timeout is reached
	' Set the function to the value (TRUE or FALSE) of the AppActivate method
	ActivateWindow = WSHShell.AppActivate(sWindow)
	If not ActivateWindow And bQuit Then
		sMessage = "Cannot activate the " & sWindow & " window. Quitting the routine."
		' Display a message for a good long time (30 seconds) but 
		' use the POPUP method so that the script can exit unattended
		iButton = WSHShell.Popup (sMessage, 30, "Pausing", 64)
		WScript.Quit
	End If
End Function
